<?php namespace awp;

abstract class ConfigModule extends Module
{
	public function __construct($pageInfo, $formData = array(), $styleHandles = array(), $scriptHandles=array())
	{
		parent::__construct();

		$formDataExample = array(
			"form1" => array(
				"group1" => array(
					"groupTitle" => "group title",
					"field1" => array( "label"=>"",  "default"=>"",  "type"=>"", "description"=>"", ),
				),
				"submit" => array(
					"text" => "",
					"action" => "", // options or custom_action 
					"settingName" => "",
				)
			)
		);


		if(!$this->RequreKeys($pageInfo, "slug", "title", "menuTitle"))
		{
			echo "Config Module Requires (object/array)pageInfo to include, these properties: slug, title and menuTitle<br/>";
			var_dump($pageInfo);
			die();
			return;
		}

		$this->ValidateFormData($formData);
		$this->pageInfo = (object)$pageInfo;
		$this->formData = $formData;
		$this->formOutput = array();
		
		$this->RequireMod("form");
		$this->RequireMod("admin-pages");

		$this->styles = $styleHandles;
		$this->scripts = $scriptHandles;

		add_action( "admin_menu", function (){ $this->CreatePage(); } );

		$formKeys = array_keys($formData);
		for($i=0; $i<count($formKeys); $i++)
		{
			$formKey = $formKeys[$i];
			$groups = $formData[$formKey];

			if(!is_array($groups))
				continue;

			$action = $groups["submit"]["action"];
			if($action != "options")
			{
				add_action( "admin_post_{$action}", function($input)use($groups){ $this->HandleSubmition($groups["submit"]["action"]);} );
			}

		}
		
		add_action("admin_enqueue_scripts", function(){
			wp_register_style( 'style-forms', AWP_URL . "styles/forms.css", array(), null );
			wp_register_style( 'style-buttons',  AWP_URL . "styles/buttons.css", array(), null );
			wp_register_style( 'style-cards',  AWP_URL . "styles/cards.css", array(), null );
			wp_register_style( 'style-tables',  AWP_URL . "styles/tables.css", array(), null );

		});


	}

	private function ValidateFormData($formData)
	{
		$formKeys = array_keys($formData);
		for($i=0; $i<count($formKeys); $i++)
		{
			$formKey = $formKeys[$i];
			$groups = $formData[$formKey];

			if(!is_array($groups))
				continue;

			$groupKeys = array_keys($groups);

			if(!$this->RequreKeys($groups, "submit") && count($groups) <= 1)
			{
				echo "<h2>Config Module Form Data Invalid</h2><br/>Must contain a group called submit and 1 aditonal group with a unique_key.";
				die();
				return;
			}

			for($j=0; $j<count($groupKeys); $j++)
			{
				$groupKey = $groupKeys[$j];
				$fields = $groups[$groupKey];
				$fieldKeys = array_keys($fields);
				
				
				if($groupKey == "submit")
				{
					if(!$this->RequreKeys( $groups["submit"], "action", "settingName"))
					{
						echo "<h2>Config Module Form Data Invalid</h2><br/>Submit Group must contain the properties: action and settingName<br/>";
						var_dump($fields["submit"]);
						die();
						return;
					}
					continue;
				}

				for($k=0; $k<count($fieldKeys); $k++)
				{
					$fieldKey =  $fieldKeys[$k];
					$filed = $fields[$fieldKey];
					if(!$this->RequreKeys( $filed, "label", "default", "type"))
					{
						echo "<h2>Config Module Form Data Invalid</h2><br/>Field must contain the properties: label, default, and type.<br/>";
						var_dump($filed);
						die();
						return;
					}
				}


			}

		}
	}

	private function RequreKeys($arrayOrObject, $string, $moreStrings=null)
	{

		return call_user_func_array(array($this,"InsertKeys"), func_get_args()) === $arrayOrObject;
	}

	private function InsertKeys($arrayOrObject, $string, $moreStrings=null)
	{
		$keys = func_get_args();
		$item = array_shift($keys);

		for($i=0; $i<count($keys); $i++)
		{
			$key = $keys[$i];
			if(is_array($item) && !isset($item[$key]))
			{
				$item[$key] = "";
			}
			else if(is_object($item) && !property_exists($item, $key))
			{
				$item->$key = "";
			}
		}
		
		return $item;
	}
	
	private function CreatePage()
	{
		$pageHook = null;

		add_action( 'admin_init', function() {$this->BuildForms();} );

		$capability = property_exists($this->pageInfo, "capability") ? $this->pageInfo->capability : "manage_options";
		$pagePosition = property_exists($this->pageInfo, "position") ? $this->pageInfo->position : null;
		$iconUrl = property_exists($this->pageInfo, "iconUrl") ? $this->pageInfo->iconUrl : "";

		$parentSlug = property_exists($this->pageInfo, "parentSlug") ? $this->pageInfo->parentSlug : null;
		$newParent = $parentSlug !== false && !isset($GLOBALS["admin_page_hooks"][$this->pageInfo->parentSlug ]);

		if($newParent)
		{
			$pageHook = add_menu_page( $this->pageInfo->title, $this->pageInfo->menuTitle, $capability, $this->pageInfo->slug, function(){$this->BuildPage();}, $iconUrl, $pagePosition );
		}
		else if( !empty($parentSlug) && array_key_exists($parentSlug, $GLOBALS["admin_page_hooks"]) )
		{
			$pageHook = add_submenu_page($parentSlug, $this->pageInfo->title, $this->pageInfo->menuTitle, $capability, $this->pageInfo->slug, function(){$this->BuildPage();}, $pagePosition );
		}
		else
		{
			//addds the page  to the options if cannot find or create parent 
			$pageHook = add_options_page( $this->pageInfo->title, $this->pageInfo->menuTitle, $capability, $this->pageInfo->slug,  function(){$this->BuildPage();}, $pagePosition);
		}

		//rename the sub-menu title. e.g. Posts is renamed to All Posts on the drop-down.
		if($newParent && property_exists($this->pageInfo, "subMenuTitle"))
		{
			add_submenu_page( $this->pageInfo->slug, $this->pageInfo->title, $this->pageInfo->subMenuTitle, $capability, $this->pageInfo->slug );
		}

		//add_action("load-{$pageHook}", function (){ $this->LoadPage(); });

		add_action("admin_enqueue_scripts", function($hook)use($pageHook){
			if($hook !== $pageHook)
				return;
			
			for($i=0; $i<count($this->styles); $i++)
			{
				wp_enqueue_style( $this->styles[$i] );
			}

			for($i=0; $i<count($this->scripts); $i++)
			{
				wp_enqueue_script( $this->scripts[$i] );
			}


		});
	}

	private function BuildPage()
	{
		?>
		<div class="wrap">
			<h1><?=esc_html_e( 'PageTitle', get_option('template') ); ?></h1>
			<div>
				<p>page info</p>
			</div>

			<?php for($i=0; $i<count($this->formOutput); $i++){
				echo $this->formOutput[$i];
			}?>
		</div>
		<?php
	}


	private function BuildForms()
	{
		$formKeys = array_keys($this->formData);

		for($i=0; $i<count($formKeys); $i++)
		{
			$formKey= $formKeys[$i];
			$groups = $this->formData[$formKey];

			if(is_callable($groups))
			{
				ob_start( ); 
					call_user_func($groups);
					$this->formOutput[] = ob_get_contents();
				ob_end_clean();
			}
			if(!is_array($groups))
				continue;

			$groupKeys = array_keys($groups);

			register_setting( $formKey, $groups["submit"]["settingName"], array("sanitize_callback"=>function($data)use($groups){ return $this->ParseInput($data, $groups);}) );
		
			for($j=0; $j<count($groupKeys); $j++)
			{
				$group = $groupKeys[$j];

				if($groupKeys[$j] != "submit")
				{
					$this->BuildForm($formKey, $group, $groups[$group], $groups["submit"]["settingName"]);

				}
			}

			$action = $groups["submit"]["action"];

			//add_action("admin_init", function() use($formKey, $action){

				ob_start( ); 
					echo "<form method=\"POST\" id=\"{$formKey}\" action=\"". ($action == "options" ? "options.php" : "admin-post.php") . "\">"; 
					$this->app->IncludeFormHeader( $formKey, $action == "options" ? "update" : $action );
					$this->app->DisplaySettingsSection( $formKey );
					submit_button();
					?></form><?php
					$this->formOutput[] = ob_get_contents();
				ob_end_clean();

			//});
		}
	}

	private function BuildForm($form, $sectionName, $fieldData, $settingName)
	{
		//print_r($sectionName);echo "<br/>";
		$groupTitle = $sectionName;
		if(array_key_exists("groupTitle", $fieldData))
		{
			$groupTitle = $fieldData["groupTitle"];
			unset($fieldData["groupTitle"]);
		}

		//register_setting( $this->slug, $this->settingName, array("sanitize_callback"=>function($data)use($fieldData){ return $this->ParseInput($data, $fieldData);}) );
		add_settings_section( $sectionName, $groupTitle, null, $form );

		//loops through fileds and loads the HTML into WP ready for page to load.
		$this->app->CreateFields($form , $fieldData, $settingName, $sectionName);
	}


	protected function HandleSubmition($action)
	{
		$sanitisedInput = $this->ValidateFormSubmition();
	}

	protected function ValidateFormSubmition($permission = "")
	{
		$permission = $permission ==  "" ? "manage_options" : $permission;
		if( !current_user_can( $permission ) && wp_verify_nonce( $_POST["_wpnonce"], $_POST["option_page"] . "-options" ) )
		{
			$this->RedirectBack();
		}

		if(!array_key_exists($_REQUEST["option_page"], $this->formData))
			return null;


		$formData = $this->formData[$_REQUEST["option_page"]];
		$settingName = $formData["submit"]["settingName"];
		//todo: check security on inputs

		$inputKeys = array_keys($_REQUEST[$settingName]);
		$sanitisedInput = array();
		
		for($i=0; $i<count($inputKeys); $i++)
		{
			$inputKey = $inputKeys[$i];
			$inputValue = $_REQUEST[$settingName][$inputKey];

			if($inputKey == "slug")
				$sanitisedInput[$inputKey] = sanitize_key($inputValue);
			else
				$sanitisedInput[$inputKey] = sanitize_text_field($inputValue);
		}

		return $sanitisedInput;
	}

	protected function RedirectBack()
	{
		if(isset($_REQUEST['_wp_http_referer']))
		{
			//var_dump(home_url());
			//wp_redirect($_REQUEST['_wp_http_referer']);
			header("Location: {$_REQUEST['_wp_http_referer']}");
			//wp_die();
			die();
		}
	}




	protected function ParseInput($inputData, $fieldData)
	{

        //NOTE this function can be called twice on the same input in options.php
        //once for add_option and once fo update_option;
        //if you are paring types you may need to do extra checks to catch errors thrown on second pass.
        //alternativly you can add checks for the action and page and return $innuptData when you get add_option from options.php

		$output = array();
		
		if($inputData == null)
			$inputData = array();

		
		$groupKeys = array_keys($fieldData);

		for($i=0; $i<count($groupKeys); $i++)
		{

			if($groupKeys[$i] == "submit")
				continue;

			$group = $fieldData[$groupKeys[$i]];
			$fieldKeys = array_keys($group);

			for($j=0; $j<count($fieldKeys); $j++)
			{
				$fieldKey = $fieldKeys[$j];
				$fieldInfo = (object)$group[$fieldKey];

				if($fieldKey == "groupTitle")
				continue;

                //var_dump($fieldKey); echo "<br/>";
                //debug_print_backtrace(0, 5);echo "<br/>";echo "<br/>";
                //var_dump($inputData); echo "<br/>";echo "<br/>";echo "<hr/>";
                
				$fieldInput = array_key_exists($fieldKey, $inputData) ? $inputData[$fieldKey]: null;

				if($fieldInput !== null)
				{	

					$fieldInput = $this->app->CheckInput($fieldInfo->type, $fieldInput);
					/*
					switch($fieldInfo->type)
					{
						case "bool":
						case "boolean":
						case "toggle":
							$fieldInput= json_decode($fieldInput); //prone to errors
							break;
						case "array":
							$fieldInput = $this->TextAreaToArray($fieldInput);
							break;
					}*/
				}
				else
				{
					$fieldInput = $fieldInfo->default;
				}

				$output[$fieldKey] = $fieldInput;

			}
		}

        //var_dump($output);
        //echo serialize (($output));
        //echo  json_encode($output);
		
		return $output;
	}

	
	private function CheckboxToBool($value)
	{
		return $value?1:0;
	}

	private function TextAreaToArray($value)
	{

        if(is_array($value))
            return $value;

		$newValue = $value;
		$newValue = sanitize_textarea_field($newValue);
		$newValue = explode("\n", $newValue);
		$newValue = array_map('trim', $newValue);

		return $newValue;
	}

}


?>