<?php namespace awp;

abstract class Module{
    protected $app;
    
    public function __construct()
    {
		$this->app = \awp\App::Instance();
    }

	protected function RequireMod($modName)
	{
        if( $this->app == null)
            $this->app = \awp\App::Instance();
		$this->app->RegisterModule($modName);
	}
}

require_once "ConfigModule.php";

require_once "classes/ConfigPage.php";

class App{

	private static $instance = null;
	private $modules;

	private $slug;
	private $version;
	private $cacheKey;
	private $cacheAllowed;
	
	
	private function __construct(){

		$this->slug = plugin_basename( __DIR__ );
		$this->version = AWP_VERSION;
		$this->cacheKey = 'ash-custom-upd';
		$this->cacheAllowed = false;

		$this->modules = array();
		
		add_filter( 'plugins_api', array( $this, 'Info' ), 20, 3 );
		add_filter( 'site_transient_update_plugins', array( $this, 'Update' ) );
		add_action( 'upgrader_process_complete', array( $this, 'Purge' ), 10, 2 );

		/*thanks https://wordpress.stackexchange.com/questions/314701/remove-view-details-option-from-plugin-listing */
		add_filter('plugin_row_meta',function ($plugin_meta, $plugin_file, $plugin_data, $status) {
			$pluginSlug = explode("/", $plugin_file)[0];
			$path = $_SERVER["REQUEST_SCHEME"] . "://" . $_SERVER["HTTP_HOST"] . "/wp-admin/" . (is_multisite()? "network/" : "");

			if($pluginSlug  == $this->slug)
				$plugin_meta[2] = "<a href=\"{$path}plugin-install.php?tab=plugin-information&plugin={$pluginSlug}&;TB_iframe=true\" class=\"thickbox open-plugin-details-modal\">View details</a>";

			return $plugin_meta;
		},10,4);

		add_action( 'admin_enqueue_scripts', function($page)
		{
			if($page == "plugin-install.php" && isset($_GET["plugin"]) && $_GET["plugin"] == dirname(plugin_basename( __FILE__ )))
            {
                wp_enqueue_style( "ash-wp-style", plugin_dir_url( __FILE__ ) . "ash-wp-style.css", array(), null);
				//wp_enqueue_style( "ash-wp-style-form", plugin_dir_url( __FILE__ ) . "styles/forms.css", array(), null);
            }
                

		});


	}

	public static function Instance()
	{
		if (self::$instance == null)
		{
			self::$instance = new app();
		}
		return self::$instance;
	}

	public function Require($file)
	{
		if(file_exists(__DIR__ . "/" .$file))
			require($file);

		else
			die("could not find file {$file}");
	}

	public function RegisterModule($moduleName)
	{

		
		$className = str_replace('-', '', ucwords(strtolower($moduleName), '-'));
		//$str = lcfirst($str);
		$className = "\\awp\\".  $className;

		$moduleFile = "modules/" . strtolower($moduleName) . "-module.php";

		if(!file_exists(AWP_PATH . $moduleFile))
			throw new \ErrorException("could not find module named: '{$moduleName}'", 0);
	
		require_once $moduleFile;
		$module = new $className();
		
		if(!is_subclass_of($module, "\\awp\\Module", false))
			throw new \ErrorException("Version must inherit from Module, for example 'class MyModule extends Module{}'", 0);

		if(!isset($this->modules[$moduleName]))
			$this->modules[$moduleName] = $module;
		
		return true;
	}

	public function __call( $method, $args )
	{
		if(!count($this->modules))
			throw new \ErrorException("No modules registerd, use \awp\App::Instance()->RegisterModule(\"moduleName\")", 0);

		$moduleNames = array_keys($this->modules);

		for($i=0; $i<count($moduleNames); $i++)
		{
			$moduleName = $moduleNames[$i];
			$classObj = $this->modules[$moduleName];
			if(method_exists($classObj, $method))
			{	
				return call_user_func_array(array($classObj, $method), $args);
			}
		}

		throw new \ErrorException("Method, $method, not found.", 0);
	}
	
	Private function Request(){

		$remote = get_transient( $this->cacheKey );

		if( false === $remote || ! $this->cacheAllowed ) {

			$remote = wp_remote_get( AWP_UPDATE_URL,
				array(
					'timeout' => 10,
					'headers' => array(
						'Accept' => 'application/json'
					)
				)
			);

			if(
				is_wp_error( $remote )
				|| 200 !== wp_remote_retrieve_response_code( $remote )
				|| empty( wp_remote_retrieve_body( $remote ) )
			) {
				return false;
			}

			set_transient( $this->cacheKey, $remote, DAY_IN_SECONDS );

		}

		$remote = json_decode( wp_remote_retrieve_body( $remote ) );

		return $remote;

	}

	public function Info($res, $action, $args )
	{

			// print_r( $action );
			// print_r( $args );

			// do nothing if you're not getting plugin information right now
			if( 'plugin_information' !== $action ) {
				return $res;
			}

			// do nothing if it is not our plugin
			if( $this->slug !== $args->slug ) {
				return $res;
			}

			// get updates
			$remote = $this->Request();

			if( ! $remote ) {
				return $res;
			}

			$res = new \stdClass();

			$res->name = $remote->name;
			$res->slug = $remote->slug;
			$res->version = $remote->version;
			$res->tested = $remote->tested;
			$res->requires = $remote->requires;
			$res->author = $remote->author;
			//$res->author_profile = $remote->author_profile;
			$res->download_link = $remote->download_url;
			$res->trunk = $remote->download_url;
			$res->requires_php = $remote->requires_php;
			$res->last_updated = $remote->last_updated;

			$res->sections = (array)$remote->sections;

			if( ! empty( $remote->banners ) ) {

				$res->banners = (array)$remote->banners;
			}

			return $res;
	}

	public function Update($transient)
	{

		if ( empty($transient->checked ) ) {
			return $transient;
		}

		$remote = $this->request();


		if(
			$remote
			&& version_compare( $this->version, $remote->version, '<' )
			&& version_compare( $remote->requires, get_bloginfo( 'version' ), '<' )
			&& version_compare( $remote->requires_php, PHP_VERSION, '<' )
		) 
		{
			$res = new \stdClass();
			$res->slug = $this->slug;
			$res->plugin = "ash-wp/ash-wp.php";
			$res->new_version = $remote->version;
			$res->tested = $remote->tested;
			$res->package = $remote->download_url;

			$transient->response[ $res->plugin ] = $res;

		}

		return $transient;
	}

	public function Purge()
	{

		if (  $this->cacheAllowed && 'update' === $options['action'] && 'plugin' === $options[ 'type' ] ) {
			// just clean the cache when new plugin version is installed
			delete_transient( $this->cacheKey );
		}
	}


	
}